/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */
#include "engine.h"

engine_t * engineInit(platform_t *platform, char *name, uint32_t inputCount) {
  // Create the engine instance.
  engine_t *engine = malloc(sizeof(engine_t));
  if(engine == NULL) return NULL;

  engine->name = name;
  engine->platform = platform;

  // Setup the renderer.
  engine->render = renderInit(name);
  if(engine->render == NULL) {
    free(engine);
    return NULL;
  }

  // Setup the input manager.
  engine->input = inputInit(inputCount, platform->inputSourceCount);
  if(engine->input == NULL) {
    free(engine->render);
    free(engine);
    return NULL;
  }

  return engine;
}

uint32_t engineUpdate(engine_t *engine) {
  renderFrame(engine->render);
  inputUpdate(engine->input);
  return 0;
}

bool engineDispose(engine_t *engine) {
  if(!renderDispose(engine->render)) return false;
  free(engine);

  return true;
}